/*
 *  jDTAUS Core RI Entity Resolver
 *  Copyright (c) 2005 Christian Schulte
 *
 *  Christian Schulte, Haldener Strasse 72, 58095 Hagen, Germany
 *  <cs@jdtaus.org> (+49 2331 3543887)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.sax.ri.resolver;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.jar.Manifest;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.logging.spi.Logger;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * {@code EntityResolver} implementation resolving XML schemas from classpath
 * resources.
 *
 * @author <a href="mailto:cs@jdtaus.org">Christian Schulte</a>
 * @version $Id: ClasspathEntityResolver.java 5138 2008-07-13 20:58:13Z schulte2005 $
 */
public final class ClasspathEntityResolver
    implements ContainerInitializer, EntityResolver
{
    //--Implementation----------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausImplementation
    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(ClasspathEntityResolver.class.getName());
// </editor-fold>//GEN-END:jdtausImplementation

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausConstructors
    // This section is managed by jdtaus-container-mojo.

    /**
     * <code>ClasspathEntityResolver</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    private ClasspathEntityResolver(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * <code>ClasspathEntityResolver</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    private ClasspathEntityResolver(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    private void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("schemaExtensions");
        this.pSchemaExtensions = (java.lang.String) p.getValue();

    }
// </editor-fold>//GEN-END:jdtausConstructors

    //------------------------------------------------------------Constructors--
    //--Dependencies------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausDependencies
    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>Logger</code> implementation. */
    private transient Logger dLogger;

    /**
     * Gets the configured <code>Logger</code> implementation.
     *
     * @return the configured <code>Logger</code> implementation.
     */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this.dLogger != null)
        {
            ret = this.dLogger;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(ClasspathEntityResolver.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(ClasspathEntityResolver.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this.dLogger = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
// </editor-fold>//GEN-END:jdtausDependencies

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausProperties
    // This section is managed by jdtaus-container-mojo.

    /**
     * Property {@code schemaExtensions}.
     * @serial
     */
    private java.lang.String pSchemaExtensions;

    /**
     * Gets the value of property <code>schemaExtensions</code>.
     *
     * @return the value of property <code>schemaExtensions</code>.
     */
    private java.lang.String getSchemaExtensions()
    {
        return this.pSchemaExtensions;
    }

// </editor-fold>//GEN-END:jdtausProperties

    //--------------------------------------------------------------Properties--
    //--ContainerInitializer----------------------------------------------------

    /**
     * Initializes the instance.
     *
     * @see #assertValidProperties
     */
    public void initialize()
    {
        this.assertValidProperties();

        try
        {
            this.schemaUrls = this.getSchemaResources();
        }
        catch ( IOException e )
        {
            this.getLogger().error(
                ClasspathEntityResolverBundle.getInstance().
                getCouldNotProcessResourcesMessage( Locale.getDefault() ).
                format( new Object[] { e.getMessage() } ) );

            this.schemaUrls = new URL[ 0 ];
        }
    }

    //----------------------------------------------------ContainerInitializer--
    //--EntityResolver----------------------------------------------------------

    public InputSource resolveEntity( final String publicId,
                                       final String systemId )
        throws SAXException, IOException
    {
        if ( systemId == null )
        {
            throw new NullPointerException( "systemId" );
        }

        InputSource schemaSource = null;

        try
        {
            final URI systemUri = new URI( systemId );
            String schemaName = systemUri.getPath();
            if ( schemaName != null )
            {
                final int lastIndexOfSlash = schemaName.lastIndexOf( '/' );
                if ( lastIndexOfSlash != -1 &&
                    lastIndexOfSlash < schemaName.length() )
                {
                    schemaName =
                        schemaName.substring( lastIndexOfSlash + 1 );

                }

                for ( int i = this.schemaUrls.length - 1; i >= 0; i-- )
                {
                    if ( this.schemaUrls[i].getPath().endsWith( schemaName ) )
                    {
                        final URI schemaUri =
                            new URI( this.schemaUrls[i].getProtocol(),
                                     this.schemaUrls[i].getPath(), null );

                        schemaSource = new InputSource();
                        schemaSource.setPublicId( publicId );
                        schemaSource.setSystemId(
                            schemaUri.toASCIIString() );

                        if ( this.getLogger().isDebugEnabled() )
                        {
                            this.getLogger().debug(
                                ClasspathEntityResolverBundle.getInstance().
                                getResolvedSystemIdMessage(
                                Locale.getDefault() ).format(
                                new Object[] { systemId,
                                               schemaSource.getSystemId()
                            } ) );

                        }

                        break;
                    }
                }
            }
            else
            {
                this.getLogger().warn(
                    ClasspathEntityResolverBundle.getInstance().
                    getUnsupportedSystemIdUriMessage( Locale.getDefault() ).
                    format( new Object[] {
                            systemId, systemUri.toASCIIString()
                        } ) );

            }
        }
        catch ( URISyntaxException e )
        {
            this.getLogger().warn(
                ClasspathEntityResolverBundle.getInstance().
                getUnsupportedSystemIdUriMessage( Locale.getDefault() ).
                format( new Object[] {
                        systemId, e.getMessage()
                    } ) );

            schemaSource = null;
        }

        return schemaSource;
    }

    //----------------------------------------------------------EntityResolver--
    //--ClasspathEntityResolver-------------------------------------------------

    /** URLs of all available classpath schema resources. */
    private URL[] schemaUrls;

    /**
     * Searches all available {@code META-INF/MANIFEST.MF} resources for
     * entries whose name end with one of the extensions specified by
     * property {@code schemaExtensions}.
     *
     * @return URLs of any matching resources.
     *
     * @throws IOException if reading or parsing fails.
     */
    private URL[] getSchemaResources() throws IOException
    {
        final ClassLoader cl = this.getClass().getClassLoader();
        final Set schemaResources = new HashSet();

        for ( Enumeration e = cl.getResources( "META-INF/MANIFEST.MF" );
            e.hasMoreElements();)
        {
            final String[] extensions = this.getSchemaExtensions().split( "," );
            final URL manifestUrl = ( URL ) e.nextElement();
            final String externalForm = manifestUrl.toExternalForm();
            final String baseUrl =
                externalForm.substring( 0, externalForm.indexOf( "META-INF" ) );

            final InputStream manifestStream = manifestUrl.openStream();
            final Manifest mf = new Manifest( manifestStream );

            manifestStream.close();

            for ( Iterator it = mf.getEntries().entrySet().iterator();
                it.hasNext();)
            {
                final Map.Entry entry = ( Map.Entry ) it.next();
                for ( int i = extensions.length - 1; i >= 0; i-- )
                {
                    if ( entry.getKey().toString().toLowerCase().
                        endsWith( '.' + extensions[i].toLowerCase() ) )
                    {
                        final URL schemaUrl =
                            new URL( baseUrl + entry.getKey().toString() );

                        schemaResources.add( schemaUrl );

                        if ( this.getLogger().isDebugEnabled() )
                        {
                            this.getLogger().debug(
                                ClasspathEntityResolverBundle.getInstance().
                                getCandidateSchemaMessage(
                                Locale.getDefault() ).format(
                                new Object[] { schemaUrl.toExternalForm() } ) );

                        }
                    }
                }
            }
        }

        return ( URL[] ) schemaResources.toArray(
            new URL[ schemaResources.size() ] );

    }

    //-------------------------------------------------ClasspathEntityResolver--
    //--ClasspathEntityResolver-------------------------------------------------

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    private void assertValidProperties()
    {
        if ( this.getSchemaExtensions() == null ||
            this.getSchemaExtensions().length() == 0 )
        {
            throw new PropertyException( "schemaExtensions",
                                         this.getSchemaExtensions() );

        }
    }

    //-------------------------------------------------ClasspathEntityResolver--
}
