/*
 *  jDTAUS Core RI Entity Resolver
 *  Copyright (c) 2005 Christian Schulte (cs@jdtaus.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.sax.ri.resolver;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;

class ClasspathEntityResolverBundle
{

  /** Singleton instance. */
  private static final ClasspathEntityResolverBundle instance =
    new ClasspathEntityResolverBundle();

  /** Creates a new {@code ClasspathEntityResolverBundle} instance. */
  ClasspathEntityResolverBundle()
  {
    this.getMessage ( "couldNotProcessResources", Locale.getDefault() );
    this.getMessage ( "candidateSchema", Locale.getDefault() );
    this.getMessage ( "unsupportedSystemIdUri", Locale.getDefault() );
    this.getMessage ( "resolvedSystemId", Locale.getDefault() );
  }

  /**
   * Gets the {@code ClasspathEntityResolverBundle} singleton.
   *
   * @return the {@code ClasspathEntityResolverBundle} singleton.
   */
  static ClasspathEntityResolverBundle getInstance()
  {
    return instance;
  }

  /**
   * Gets the text of the message with key {@code "couldNotProcessResources"}.
   *
   * @return <pre>
   * Could not process classpath resources. Resolver is disabled ! {0}</pre>
   */
  String getCouldNotProcessResourcesText ( final Locale locale )
  {
    return this.getMessage ( "couldNotProcessResources", locale );
  }

  /**
   * Gets the message with key {@code "couldNotProcessResources"}.
   *
   * @return <pre>
   * Could not process classpath resources. Resolver is disabled ! {0}</pre>
   */
  MessageFormat getCouldNotProcessResourcesMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "couldNotProcessResources", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "candidateSchema"}.
   *
   * @return <pre>
   * Added {0} to the list of candidate schema resources.</pre>
   */
  String getCandidateSchemaText ( final Locale locale )
  {
    return this.getMessage ( "candidateSchema", locale );
  }

  /**
   * Gets the message with key {@code "candidateSchema"}.
   *
   * @return <pre>
   * Added {0} to the list of candidate schema resources.</pre>
   */
  MessageFormat getCandidateSchemaMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "candidateSchema", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "unsupportedSystemIdUri"}.
   *
   * @return <pre>
   * Unsupported system id URI "{0}". {1}</pre>
   */
  String getUnsupportedSystemIdUriText ( final Locale locale )
  {
    return this.getMessage ( "unsupportedSystemIdUri", locale );
  }

  /**
   * Gets the message with key {@code "unsupportedSystemIdUri"}.
   *
   * @return <pre>
   * Unsupported system id URI "{0}". {1}</pre>
   */
  MessageFormat getUnsupportedSystemIdUriMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "unsupportedSystemIdUri", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "resolvedSystemId"}.
   *
   * @return <pre>
   * {0}
    -> {1}</pre>
   */
  String getResolvedSystemIdText ( final Locale locale )
  {
    return this.getMessage ( "resolvedSystemId", locale );
  }

  /**
   * Gets the message with key {@code "resolvedSystemId"}.
   *
   * @return <pre>
   * {0}
    -> {1}</pre>
   */
  MessageFormat getResolvedSystemIdMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "resolvedSystemId", locale ), locale );
  }

  /**
   * Gets a message for a given key and locale.
   *
   * @param key the key of the message to return.
   * @param locale the locale of the message.
   *
   * @return message for {@code key} using {@code locale}.
   */
  private String getMessage( final String key, Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return ResourceBundle.getBundle( "org/jdtaus/core/sax/ri/resolver/ClasspathEntityResolver",
                                     locale ).getString( key );

  }
}
